<?php
/**
 * @package   AkeebaLoginGuard
 * @copyright Copyright (c)2016-2022 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU General Public License version 3, or later
 */

namespace Akeeba\Component\LoginGuard\Administrator\Controller;

// Prevent direct access
defined('_JEXEC') or die;

use Akeeba\Component\LoginGuard\Administrator\Controller\Mixin\ControllerEvents;
use Akeeba\Component\LoginGuard\Administrator\Controller\Mixin\GetRedirectionAware;
use Akeeba\Component\LoginGuard\Administrator\Controller\Mixin\RegisterControllerTasks;
use Akeeba\Component\LoginGuard\Administrator\Controller\Mixin\ReturnURLAware;
use Akeeba\Component\LoginGuard\Administrator\Controller\Mixin\ReusableModels;
use Akeeba\Component\LoginGuard\Administrator\Event\NotifyActionLog;
use Akeeba\Component\LoginGuard\Administrator\Helper\Mfa;
use Akeeba\Component\LoginGuard\Administrator\Helper\Mfa as MfaHelper;
use Akeeba\Component\LoginGuard\Administrator\Model\ConvertModel;
use Exception;
use Joomla\CMS\Application\CMSApplication;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\User\User;
use Joomla\CMS\User\UserFactoryInterface;
use Joomla\Input\Input;
use RuntimeException;

class ConvertController extends BaseController
{
	use ControllerEvents;
	use GetRedirectionAware;
	use RegisterControllerTasks;
	use ReturnURLAware;
	use ReusableModels;

	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		$this->registerDefaultTask('Convert');
	}

	public function convert($cachable = false, $urlparams = false)
	{
		$this->assertSuperUser();

		// Set up the Model and View
		/** @var ConvertModel $model */
		$model = $this->getModel();
		$view  = $this->getView();

		$view->setModel($model, true);

		// Perform the conversion
		$result = $model->convert();

		// Set the correct layout depending on what happened to the conversion
		$view->setLayout('default');

		if (!$result)
		{
			$view->setLayout('done');
			$model->disableTFA();

			MfaHelper::triggerEvent(
				new NotifyActionLog('onComLoginguardControllerConvertAfterConvert', [])
			);
		}

		// Render the view
		$view->display();

		return $this;
	}

	/**
	 * Assert that the user is a Super User
	 *
	 * @param   User|null  $user  The user to assert. Null to use the currently logged in user
	 *
	 * @return  void
	 * @throws  Exception
	 */
	protected function assertSuperUser(User $user = null)
	{
		if (empty($user))
		{
			$user = $this->app->getIdentity()
				?: Factory::getContainer()->get(UserFactoryInterface::class)->loadUserById(0);

		}

		if ($user->authorise('core.admin'))
		{
			return;
		}

		throw new RuntimeException(Text::_('JERROR_ALERTNOAUTHOR'), 403);
	}
}