<?php
/**
 * @package   AkeebaLoginGuard
 * @copyright Copyright (c)2016-2022 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU General Public License version 3, or later
 */

namespace Akeeba\Component\LoginGuard\Administrator\Event;

defined('_JEXEC') or die;

use Akeeba\Component\LoginGuard\Administrator\Event\Backport\ResultAware;
use Akeeba\Component\LoginGuard\Administrator\Event\Backport\ResultAwareInterface;
use Akeeba\Component\LoginGuard\Administrator\Event\Backport\ResultTypeArrayAware;
use Akeeba\Component\LoginGuard\Administrator\Table\TfaTable;
use DomainException;
use Joomla\CMS\Event\AbstractImmutableEvent;
use Joomla\Input\Input;

/**
 * Concrete Event class for the onLoginGuardMfaSaveSetup event
 *
 * @package     Akeeba\Component\LoginGuard\Administrator\Event
 *
 * @since       6.0.0
 */
class SaveSetup extends AbstractImmutableEvent implements ResultAwareInterface
{
	use ResultAware;
	use ResultTypeArrayAware;

	/** @inheritdoc */
	public function __construct(TfaTable $record, Input $input)
	{
		parent::__construct('onLoginGuardMfaSaveSetup', [
			'record' => $record,
			'input'  => $input,
		]);

		$this->resultIsNullable = true;
	}

	/**
	 * Validate the value of the 'record' named parameter
	 *
	 * @param   TfaTable  $value
	 *
	 * @return  TfaTable
	 * @since   6.0.0
	 */
	public function setRecord(TfaTable $value): TfaTable
	{
		if (empty($value))
		{
			throw new DomainException(sprintf('Argument \'record\' of event %s must be a TfaTable object', $this->name));
		}

		return $value;
	}

	/**
	 * Validate the value of the 'record' named parameter
	 *
	 * @param   Input  $value
	 *
	 * @return  Input
	 * @since   6.0.0
	 */
	public function setInput(Input $value): Input
	{
		if (empty($value))
		{
			throw new DomainException(sprintf('Argument \'input\' of event %s must be an Input object', $this->name));
		}

		return $value;
	}
}